@@ -18,7 +18,7 @@ This product includes software developed by and/or derived from the Access
 Grid Project (http://www.accessgrid.org) to which the U.S. Government retains 
 certain rights."""
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = '$Id: $'
+__revision__ = '$Id: client.py 7772 2010-12-09 13:40:45Z pjkersha $'
 import logging
 log = logging.getLogger(__name__)
 
@@ -53,15 +53,13 @@ class MyProxyServerSSLCertVerification(object):
     }
     PARSER_RE_STR = '/(%s)=' % '|'.join(DN_LUT.keys() + DN_LUT.values())
     PARSER_RE = re.compile(PARSER_RE_STR)    
-        
+    SERVER_CN_PREFIXES = ('host/', 'myproxy/', '')
+    
     SERVER_CN_PREFIX = 'host/'
 
-    __slots__ = ('__hostname', '__cnPrefix', '__certDN')
+    __slots__ = ('__hostname', '__certDN')
     
-    def __init__(self, 
-                 certDN=None,
-                 hostname=None,
-                 cnPrefix=SERVER_CN_PREFIX):
+    def __init__(self, certDN=None, hostname=None):
         """Override parent class __init__ to enable setting of certDN
         setting
         
@@ -70,7 +68,6 @@ class MyProxyServerSSLCertVerification(object):
         MyProxy server to avoid errors matching hostnames.  This is useful
         where the hostname is not fully qualified
         """
-        self.__cnPrefix = None
         self.__certDN = None
         self.__hostname = None
         
@@ -79,11 +76,9 @@ class MyProxyServerSSLCertVerification(object):
             
         if hostname is not None:
             self.hostname = hostname
-            
-        self.cnPrefix = cnPrefix
         
     def __call__(self, connection, peerCert, errorStatus, errorDepth, 
-                 successStatus):
+                 preverifyOK):
         """Verify MyProxy server certificate
         
         @type connection: OpenSSL.SSL.Connection
@@ -92,13 +87,23 @@ class MyProxyServerSSLCertVerification(object):
         @param peerCert: MyProxy server host certificate as OpenSSL.crypto.X509
         instance
         @type errorStatus: int
-        @param errorStatus: error code to return if verification fails
+        @param errorStatus: error status passed from caller.  This is the value
+        returned by the OpenSSL C function X509_STORE_CTX_get_error().  Look-up
+        x509_vfy.h in the OpenSSL source to get the meanings of the different
+        codes.  PyOpenSSL doesn't help you!
         @type errorDepth: int
-        @param errorDepth: 
-        @type successStatus: int
-        @param successStatus: 
+        @param errorDepth: a non-negative integer representing where in the 
+        certificate chain the error occurred. If it is zero it occured in the 
+        end entity certificate, one if it is the certificate which signed the 
+        end entity certificate and so on.
+
+        @type preverifyOK: int
+        @param preverifyOK: the error status - 0 = Error, 1 = OK of the current
+        SSL context irrespective of any verification checks done here.  If this
+        function yields an OK status, it should enforce the preverifyOK value
+        so that any error set upstream overrides and is honoured.
         @rtype: int
-        @return: status code
+        @return: status code - 0/False = Error, 1/True = OK
         """
         if peerCert.has_expired():
             # Any expired certificate in the chain should result in an error
@@ -121,22 +126,24 @@ class MyProxyServerSSLCertVerification(object):
                               'certificate against')
                     return False
                     
-                cn = self.cnPrefix + self.hostname
-                if peerCertSubj.commonName == cn:
-                    return True
+                acceptableCNs = [pfx + self.hostname 
+                                 for pfx in self.__class__.SERVER_CN_PREFIXES]
+                if peerCertSubj.commonName in acceptableCNs:
+                    return preverifyOK
                 else:
                     log.error('Peer certificate CN %r doesn\'t match the '
-                              'expected CN %r', peerCertSubj.commonName, cn)
+                              'expected CN %r', peerCertSubj.commonName, 
+                              acceptableCNs)
                     return False
             else:
                 if peerCertDN == self.certDN:
-                    return True
+                    return preverifyOK
                 else:
                     log.error('Peer certificate DN %r doesn\'t match the '
                               'expected DN %r', peerCertDN, self.certDN)
                     return False
         else:
-            return True
+            return preverifyOK
              
     def _getCertDN(self):
         return self.__certDN
@@ -181,23 +188,6 @@ class MyProxyServerSSLCertVerification(object):
     hostname = property(fget=_getHostname,
                         fset=_setHostname,
                         doc="hostname of MyProxy server")
-    
-    def _getCNPrefix(self):
-        """References SSL Certificate verification object property!"""
-        return self.__cnPrefix
-    
-    def _setCNPrefix(self, val):
-        """Sets SSL Certificate verification object property!"""
-        if not isinstance(val, basestring):
-            raise TypeError("Expecting string type for hostname "
-                                 "attribute")
-        self.__cnPrefix = val
-    
-    cnPrefix = property(fget=_getCNPrefix,
-                        fset=_setCNPrefix,
-                        doc="Prefix for MyProxy Server Certificate "
-                            "Distinguished Name CommonName field; usually set "
-                            "to 'host/' for Globus host certificates")
                     
     
 class MyProxyClientError(Exception):
@@ -318,6 +308,12 @@ class MyProxyClient(object):
     @type X509_CERT_DIR_ENVVARNAME: string
     @param X509_CERT_DIR_ENVVARNAME: environment variable name 'X509_CERT_DIR',
     which if set points to the location of the trust roots 
+    
+    @type X509_USER_PROXY_ENVVARNAME: string
+    @param X509_USER_PROXY_ENVVARNAME: environment variable name 
+    'X509_USER_PROXY' if set points to the output location of the output EEC /
+    Proxy certificate.  Not currently used by this class, included for
+    reference only
     """
     MYPROXY_SERVER_ENVVARNAME = 'MYPROXY_SERVER'
     MYPROXY_SERVER_PORT_ENVVARNAME = 'MYPROXY_SERVER_PORT'
@@ -393,7 +389,6 @@ TRUSTED_CERTS=1"""
     PROPERTY_DEFAULTS = {
        'hostname':              'localhost',
        'port':                  7512,
-       'cnPrefix':          MyProxyServerSSLCertVerification.SERVER_CN_PREFIX,
        'serverDN':              None,
        'openSSLConfFilePath':   '',
        'proxyCertMaxLifetime':  43200,
@@ -405,6 +400,7 @@ TRUSTED_CERTS=1"""
     ROOT_TRUSTROOT_DIR = '/etc/grid-security/certificates'
     USER_TRUSTROOT_DIR = '~/.globus/certificates'    
     X509_CERT_DIR_ENVVARNAME = 'X509_CERT_DIR'
+    X509_USER_PROXY_ENVVARNAME = 'X509_USER_PROXY'
     
     # Restrict attributes to the above properties, their equivalent 
     # protected values + extra OpenSSL config object.
@@ -571,21 +567,6 @@ TRUSTED_CERTS=1"""
                         fset=_setServerDN,
                         doc="Distinguished Name for MyProxy Server "
                             "Certificate")
-    
-    def _getServerCNPrefix(self):
-        """References SSL Certificate verification object property!"""
-        return self.__serverSSLCertVerify.cnPrefix
-    
-    def _setServerCNPrefix(self, val):
-        """Sets SSL Certificate verification object property!"""
-        self.__serverSSLCertVerify.cnPrefix = val
-    
-    serverCNPrefix = property(fget=_getServerCNPrefix,
-                              fset=_setServerCNPrefix,
-                              doc="Prefix for MyProxy Server Certificate "
-                                  "Distinguished Name CoomonName field; "
-                                  "usually set to host/ for Globus host "
-                                  "certificates")
         
     def _getOpenSSLConfFilePath(self):
         return self.__openSSLConfFilePath
@@ -706,9 +687,9 @@ TRUSTED_CERTS=1"""
         
         if verifyPeerWithTrustRoots:
             context.load_verify_locations(None, self.caCertDir)
-            
-        # Verify peer's (MyProxy server) certificate
-        context.set_verify(SSL.VERIFY_PEER, self.__serverSSLCertVerify)
+                
+            # Verify peer's (MyProxy server) certificate
+            context.set_verify(SSL.VERIFY_PEER, self.__serverSSLCertVerify)
              
         if certFile:
             try:
@@ -803,7 +784,7 @@ TRUSTED_CERTS=1"""
         
         # get response value
         respCode = [int(v) for k, v in fields if k == 'RESPONSE'][0]
-
+        
         # get error text
         errorTxt = os.linesep.join([v for k, v in fields if k == 'ERROR'])
         
@@ -1370,6 +1351,17 @@ TRUSTED_CERTS=1"""
         
         # - n certs
         dat = conn.recv(MyProxyClient.SERVER_RESP_BLK_SIZE)
+
+        # Check for error generating certificate
+        try:
+            respCode, errorTxt = self._deserializeResponse(dat)
+            if respCode:
+                raise MyProxyClientGetError(errorTxt)
+            
+        except ValueError:
+            # If all is well, no error response will have been set so an attempt
+            # to parse an error code will fail
+            pass
     
         # process server response
         resp = conn.recv(MyProxyClient.SERVER_RESP_BLK_SIZE)
